/* Common */
html, body {height: 100%;}
body {font-family: Arial;}
ul {list-style: none; padding: 0px;}

/* Icons */
.caption i.fa {width: 25px;}
.caret {margin: 4px 0px 8px 8px}

/* Helpers */
.hidden {display: none;}
.relative {position: relative;}
.fixed {position: fixed;}
.stretched {width: 100%;}

/* Forms */
.form-control {border-radius: 5px;}
.form-control[readonly] {background-color: inherit; cursor: context-menu;}
.form-group {position: relative;}

/* Lists */
.list-panel {padding-top: 5px;}
.list-panel input {margin: 10px 0px;}
.list-panel button {width: 100%; margin-bottom: 10px;}
.list-panel i.fa {margin-top: 5px;}
.pager li span {margin: 10px;}

/* Tables */
td {vertical-align: middle !important;}
@media screen and (min-width: 992px) {tr {height: 40px;}}
